#!/usr/bin/python
# -*- coding: utf-8 -*-
#
import feedparser
import sys, time, datetime, getopt, os

FeedURI = "feed.xml"

_dateFormat = "%Y-%m-%dT%H:%M+00:00"


#################################################
_feedStart1 = u"""<?xml version="1.0" encoding="utf-8" ?>
<rdf:RDF 
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns="http://purl.org/rss/1.0/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:content="http://purl.org/rss/1.0/modules/content/">
    <channel rdf:about="%s">
        <title>%s</title>
        <link>%s</link>
        <description>%s</description>
        <dc:source>%s</dc:source>
        <dc:date>%s</dc:date>
 """
_feedStart2 = u"""
        <dc:creator>%s</dc:creator>
        <dc:rights>Copyright © %s %s</dc:rights>
"""
_feedStart3 = u"""
        <items>
            <rdf:Seq>
"""
_feedTOC='                    <rdf:li rdf:resource="%s"/>\n'
_feedTOCEnd="""             </rdf:Seq>
        </items>
"""
_feedHeaderEnd="""    </channel>
"""

class toRss1Converter :
	def __init__(self,author,title,feed,filter,authorFilter,outp) :
		"""
		@param author: the author of the feed
		@param feed: the feed structure as generated by the feeparser
		@param filter: array of (string) terms
		@param authorFilter: possible author name to filter against
		@param outp: output stream for the result	
		"""
		self.author  = _sanitize(author)
		self.feed    = feed
		self.title   = title
		self.outp    = outp
		self.entries = self._filterAuthor(self._filterItems(feed.entries,filter),authorFilter)
		if self.entries :
			(self.maxDate,self.year) = self._maxDate()
		else :
			try :
				# there are no entries! Just take the date in the initial feed...
				self.maxDate = time.strftime(_dateFormat,feed.feed.updated_parsed)
				self.year    = feed.feed.updated_parsed[0]
			except :
				# not even a date...
				(self.maxDate,self.year) = ("2000-01-01T00:00+00:00",2000)

	def startup(self,link) :
		"""
		The header part of the rss1.0 feed is generated, including the <Seq> for the table of content. 
		
		The date is calculated by taking the maximum of all the dates appearing in the entries, and 
		not the date of the incoming feed. The reason is that filtering may filter out some of the 
		entries that sets the date of the full feed.
		
		@param link: the link to be put into the <link> statement of the feed header
		"""
		header = _feedStart1 % (link,self.title,link,self.title,link,self.maxDate)
		if self.author :
			header += _feedStart2 % (self.author,self.year,self.author) 
		self.outp.write(header.encode('utf-8'))
		if self.entries :
			self.outp.write(_feedStart3)
			for entry in self.entries :
				self.outp.write(_feedTOC % entry.link)
			self.outp.write(_feedTOCEnd)
		self.outp.write(_feedHeaderEnd)

	def writeItems(self) :
		"""
		Generate all individual items
		"""
		for i in self.entries: self._writeItem(i)
		
	def end(self) :
		"""
		Close the rss1.0 feed by closing the XML statements.
		"""
		self.outp.write("</rdf:RDF>\n")

	#################################################
	def _filterItems(self,entries,filter=None) :
		"""
		Filter entries. The 'entries' is an array of entries in feedparser's entry format; filter is an array of
		accepted terms. The entry is accepted if one of its 'term' field appears in the filter array.
		
		@param entries: array of entries in feedparser's entry format
		@param filter: array of (string) terms
		@return: array of entries in feedparser's entry format
		"""
		retval = []
		for entry in entries :
			if filter : 
				added = False
				for tag in entry.tags :
					if tag['term'] in filter and added == False :
						retval.append(entry)
						added = True
			else :
				retval.append(entry)
		return retval
		
	#################################################
	def _filterAuthor(self,entries,author=None) :
		"""
		Filter entries as author: only those entries will be allowed that has 'author' as, well, author 
		
		@param entries: list of entries
		@param author: author name to filter against
		@return: array of entries in feedparser's entry format
		"""
		if not author : return entries
		author = _sanitize(author)
		retval = []
		for entry in entries :
			try :
				entryAuthor = _sanitize(entry.author)
				if entryAuthor == author :
					retval.append(entry)						
			except :
				pass
		return retval

	#################################################
	def _writeItem(self,entry) :
		"""
		Write a single entry.
		
		@param entry: and entry in feedparser's entry format
		"""
		indent = "        "
		self.outp.write('    <item rdf:about="%s">\n' % entry.link)
		self.outp.write(indent + ('<title>%s</title>' % _sanitize(entry.title)).encode('utf-8') + '\n')
		self.outp.write(indent + ('<dc:subject>%s</dc:subject>' % _sanitize(entry.title)).encode('utf-8') + '\n')
		self.outp.write(indent + ('<link>%s</link>' % entry.link) + '\n')
		date = time.strftime(_dateFormat,entry.updated_parsed)
		self.outp.write(indent + ('<dc:date>%s</dc:date>' % date)+'\n')		
		author = self.author
		
		try :
			author = _sanitize(entry.author).encode('utf-8')
		except :
			pass
		if author :
			self.outp.write(indent + ('<dc:creator>%s</dc:creator>' % author) + '\n')
			self.outp.write(indent + ('<dc:author>%s</dc:author>' % author) + '\n')
			cpright = (u'<dc:rights>Copyright © %s %s</dc:rights>').encode('utf-8')
			self.outp.write(indent + (cpright % (entry.updated_parsed[0],author)) + '\n')

		for tag in entry.tags :
			self.outp.write(indent + ('<dc:subject>%s</dc:subject>' % tag['term']).encode('utf-8') + '\n')
			
		try :
			# note: some feeds return xml data in the summary, and the feedparser turns that into real xml. That has to be 
			# escaped into a CDATA
			if entry.summary.find('<') != -1 :
				pattern = '<description><![CDATA[%s]]></description>'
				summary = entry.summary
			else :
				pattern = '<description>%s</description>'
				summary = _sanitize(entry.summary)
			self.outp.write(indent + (pattern % summary).encode('utf-8') + '\n')
		except :
			pass
		
		try :
			self.outp.write(indent + ('<content:encoded><![CDATA[%s]]></content:encoded>' % entry.content[0]['value']).encode('utf-8') + '\n')
		except :
			pass
		self.outp.write('    </item>\n')

	def _maxDate(self) :
		"""
		Calculate the maximum dates for a set of entries.
		
		@return: a tuple with (1) the date in ISO date format as a string and (2) year 
		"""
		maxDate = datetime.datetime(2000,01,01)
		retval  = ("2000-01-01T00:00+00:00",2000)
		for entry in self.entries :
			try:
				edate = datetime.datetime(entry.updated_parsed[0],
											entry.updated_parsed[1],
											entry.updated_parsed[2],
											entry.updated_parsed[3],
											entry.updated_parsed[4]
				)
				if edate > maxDate :
					maxDate = edate
					retval  = (time.strftime(_dateFormat,entry.updated_parsed),entry.updated_parsed[0])
			except :
				pass
		return retval

#################################################
def _sanitize(txt) :
	"""
	Sanitize strings: exchange all '&' into '&amp;'. Unfortunately I have hit feeds that do not do that,
	ie, they are invalid XML-s :-(
	@param txt: string
	@return: sanitized version
	"""
	return txt.replace('&','&amp;')
#################################################
def _createFilter(arguments) :
	"""
	Creation of a filter array for the rest of the operations. Essentially, turning a string list of words into an
	array of words (the former being more adequate for command line usage). A name of the form [a b] 
	is considered to be one category.
	
	@param arguments: an argument dictionary; the relevant key is 'categories'
	@retval: array of strings for the filter
	"""
	if "categories" in arguments :
		categories = arguments["categories"]
		retval = []
		while True :
			lb = categories.find('[')
			if lb == -1 :
				# no compound category
				break
			# if we got here, we have a compound category, i.e., surrounded by brackets. Let us find the end of it:
			rb = categories.find(']',lb)
			if rb == -1 :
				# we have an illegal set of arguments, the best is just to break again
				break
			# slice the categories out, stick the rest together
			slice = categories[lb+1:rb]
			retval.append(slice.strip())
			newCategories = categories[:lb] + ' ' + categories[rb+1:]
			categories = newCategories
		# The rest is simple: just split it at the spaces
		return retval + categories.split(None)
	else :
		return []

#################################################
def main(outp,form) :
	"""
	@param outp: output stream for the result	
	@param form: argument dictionary
	"""
	# first get the uri. If this is not specified, the whole process stops...
	uri = None
	try :
		uri = form["uri"]
	except :
		return
	# get the (possible) filters
	filter = _createFilter(form)
	# see if a separate link has been specified; otherwise the uri is used
	if "link" in form :
		link = form["link"]
	else :
		link = uri
	# read in the full feed
	feed = feedparser.parse(uri)
	# set the author. Priority: if there is a name in the argument, that wins; then the feed is looked for an author. Default: fall back on empty...
	author = ""
	if "author" in form :
		author = form["author"]
	else :
		try :
			author = feed.feed.author
		except :
			pass
			
	# feed title. If there is a title in the argument, that wins; otherwise, the title in the feed wins. Default: fall back on empty...
	title = ""
	if "title" in form :
		title = form["title"]
	else :
		try :
			title = _sanitize(feed.feed.title)
		except :
			pass
			
	if "authorFilter" in form :
		authorFilter = form["authorFilter"]
	else :
		authorFilter = None
	converter = toRss1Converter(author,title,feed,filter,authorFilter,outp)
	converter.startup(link)
	converter.writeItems()
	converter.end()
	
##################################################

usageTxt = """%s [options]
	-a author (overrides author name in feed)
	-c categories (list of words acting as filters; multiple words can be put in [])
	-u feed uri (this is a required argument)
	-l feed link (link to be used for the feed output, if different from feed uri)
	-t title of the feed (overrides the title in the feed)
	-r filter on the creator of the entry
	-h this message
	-o output file name (default is standard output)
	
"""

def _usage() :
	print usageTxt % sys.argv[0]

def commandLine() :
	retval = {}
	try :
		(opts,args) = getopt.getopt(sys.argv[1:],"a:c:u:l:h:r:t:")
		
		for t in opts :
			# Help
			if t[0] == "-a" :
				retval["author"] = t[1]
			if t[0] == "-c" :
				retval["categories"] = t[1]
			if t[0] == "-u" :
				retval["uri"] = t[1]
			if t[0] == "-l" :
				retval["link"] = t[1]
			if t[0] == "-t" :
				retval["title"] = t[1]				
			if t[0] == "-o" :
				retval["output"] = file(t[1],"w")
			if t[0] == "-r" :
				retval["authorFilter"] = t[1]
			if t[0] == "-h" :
				_usage()
				sys.exit(0)
		if "uri" in retval :
			return retval
	except :
		pass
	_usage()
	sys.exit(1)

# For testing
arguments = {
	"author"     : "Ivan Herman",
	"categories" : "lopikula [cintanyer es lopikula] [Semantic Web] RDFLib",
	"uri"        : "feed.xml",
	"link"       : "blabla.xml"
}
if __name__ == '__main__' :
	args = commandLine()
	if "output" in args :
		outp = args["output"]
	else :
		outp = sys.stdout
	main(outp,commandLine())
